#-*- coding:utf-8 -*-
# ----------------------------------------------
# Antenna B Serial Port & MCU Serial Port
# ----------------------------------------------
#                               [1152000bps]     
#   Antenna B Serial Port <-> MCU <-> RaspberryPI
#                      [57600bps]                  
# ----------------------------------------------
import serial
import time
from Antenna import *
from NMEA import *
from MCU import *
from Communicator import *
from threading import Thread

class McuCommmunicator(Communicator):
    def __init__(self,name):
        Communicator.__init__(self,name)
        self.Ant = Antenna('Antenna')
        self.Nmea = NmeaFormatter()
        self.Mcu = MCU('MCU')
        self.DevName = ''
        Communicator.ResetFlag(self)
        self.SubAnt = Communicator("SubAnt")
        self.SubAnt.ResetFlag()
        self.TxArr = []
        
        self.Connected = 0
        
        p = Thread(target=self.TxWorker)
        p.start()
        p1 = Thread(target=self.SubWorker)
        p1.start()
        self.NonRcvCnt = 0
        self.BaudrateFlag = 0

    def Open(self,name,baudrate):
        self.out("Opend %s-%s"%(name,baudrate))
        self.DevName = name
        self.Comm = serial.Serial(name, baudrate)
    def SetBaudrate(self,name,baudrate):
        self.Open(self.DevName, baudrate)
    def Close(self):
        self.Comm.close() 
    def Start(self):
        line = ""
        while True: 
            ch  = self.Comm.read(1)
            line += ch    
            if ch == '\n':
				self.DatProc(line)
				line = ""	
    def DatProc(self,line):
        # self.out(line)
        if len(line) < 5 : return 
        if line[0:2] == '$K' :
            lines = line.split(',')
            # print line
            if len(lines) > 3 : 
                if lines[2] == 'STS' : 
                    self.SubAnt.SetFlag()
                    self.Ant.DatProc(line) 
                    # self.out(line)
                elif lines[2] == 'GPS' : 
                    self.Ant.Nmea.BurstGpsProc(line)
                    self.out(line)
                elif lines[1] == 'SYS' : 
                    self.Mcu.BurstGpsProc(line) 
                    # self.out(line)
        elif line[0:2] == '$G' or line[0:2] == '$H' : 
            self.out(line)
            self.Nmea.DatProc(line)
        self.SetFlag()
    def Send(self,dat):
        self.TxArr.append(dat)
    def SendAttn(self,idx,lvl):
        tmp = "$KW,ATTN,%d,%.2f,"%(idx, lvl)
        self.Send(Util.AppendCRC(tmp))
    def SendAntSelect(self,idx):
        tmp = "$KW,ANTSEL,%d,"%(idx)
        self.Send(Util.AppendCRC(tmp))
    def SendRefFreqSelect(self,idx):
        tmp = "$KW,FRQSEL,%d,"%(idx)
        self.Send(Util.AppendCRC(tmp))
    def SendRefFreqManual(self,idx0,idx1,idx2):
        tmp = "$KW,FRQSEL_MNL,%d,%d,%d,"%(idx0,idx1,idx2)
        self.Send(Util.AppendCRC(tmp))
    def SendMacAddr(self,d0,d1,d2,d3,d4,d5):
        tmp = "$KW,MAC,%d,%d,%d,%d,%d,%d,"%(d0,d1,d2,d3,d4,d5) 
        self.Send(Util.AppendCRC(tmp))
    def SendReboot(self,dat):
        # "$KW,REBOOT,
        self.Send(tmp)
    def TxWorker(self):
        time.sleep(2)
        while True:
            time.sleep(0.07) 
            if len(self.TxArr) > 0 : 
                dat = self.TxArr.pop(0)
                # self.out("len {}  {}".format(len(self.TxArr),dat ))
                try:
                    lines = dat.split(',')
                    if lines[1] == "DAT_O" : self.out(dat)
                    elif lines[1] == "DAT_P" : self.out(dat)
                    elif lines[0] == "$KW" : self.out(dat)
                    
                    
                    self.Comm.write(dat)
                except: self.out('Transfer Error - Antenna AntennaSubCommunicator')
                
    def SubWorker(self): 
        time.sleep(1)
        from Project import pj 
        while True:
            time.sleep(0.2)
            if self.oRcvCnt != self.RcvCnt:
                self.oRcvCnt = self.RcvCnt
                self.NonRcvCnt = 0
                self.Connected = 1
                break;
            else :
                self.NonRcvCnt = self.NonRcvCnt + 1 
                
                if self.NonRcvCnt > 6 :
                    self.NonRcvCnt = 0
                    if self.BaudrateFlag == 1 : 
                        self.SetBaudrate(pj.Sys.GetMcuName(), 576000)
                        self.BaudrateFlag = 0
                        self.out("Changed baudrate 1152000 >> 576000")
                    else :
                        self.SetBaudrate(pj.Sys.GetMcuName(),1152000)
                        self.BaudrateFlag = 1
                        self.out("Changed baudrate 576000 >> 115200")
                        
                
            


